/*
 * Copyright (c) 2018 WIT Software. All rights reserved.
 *
 * WIT Software Confidential and Proprietary information. It is strictly forbidden for 3rd parties
 * to modify, decompile,
 * disassemble, defeat, disable or circumvent any protection mechanism; to sell, license, lease,
 * rent, redistribute or
 * make accessible to any third party, whether for profit or without charge.
 */

package com.witsoftware.wmc.addons.clevertap.handlers

import com.wit.wcl.URI
import com.wit.wcl.URIUtils
import com.witsoftware.wmc.application.ui.BaseActivity
import com.witsoftware.wmc.addons.clevertap.cb.ICleverTapSessionListener
import com.witsoftware.wmc.addons.clevertap.entities.CleverTapSession
import com.witsoftware.wmc.addons.clevertap.utils.CleverTapValues
import com.witsoftware.wmc.appstate.AppStateManager
import com.witsoftware.wmc.appstate.cb.IAppStateChangedListener
import com.witsoftware.wmc.chatbots.ChatbotsUtils
import com.witsoftware.wmc.chats.ChatManager
import com.witsoftware.wmc.chats.cb.ICurrentChatListener
import com.witsoftware.wmc.chats.ui.BaseChatActivity
import com.witsoftware.wmc.logs.Lucy
import com.witsoftware.wmc.threads.utils.doInBackground
import java.util.concurrent.TimeUnit

class CleverTapSessionHandler(private val sessionListener: ICleverTapSessionListener) : ICurrentChatListener, IAppStateChangedListener {

    private val TAG = "CleverTapSessionHandler"

    private val activeSessionLock = Unit
    private var activeSession: CleverTapSession? = null

    fun init() {
        Lucy.d(TAG, "init")

        ChatManager.getInstance().subscribeCurrentChanged(this)
        AppStateManager.getInstance().subscribeAppStateChangedEvents(this)
    }

    fun destroy() {
        Lucy.d(TAG, "destroy")

        ChatManager.getInstance().unsubscribeCurrentChanged(this)
        AppStateManager.getInstance().unsubscribeAppStateChangedEvents(this)
        clearSession()
    }

    override fun onCurrentChatChanged(uri: URI?) {
        Lucy.d(TAG, "onCurrentChatChanged | uri=$uri")

        doInBackground {
            if (uri == null) {
                endSession()

            } else {
                startSession(uri)
            }
        }
    }

    override fun onBackgroundState() {
        Lucy.d(TAG, "onBackgroundState")
        endSession()
    }

    override fun onForegroundState() {
        // Do nothing
    }

    private fun startSession(currentChatUri: URI) {
        Lucy.d(TAG, "startSession | currentChatUri=$currentChatUri")

        synchronized(activeSessionLock) {
            BaseActivity.getCurrentActivity() as? BaseChatActivity ?: return

            val chatBot = ChatbotsUtils.getChatbot(currentChatUri) ?: return

            val chatBotServiceId: String? = chatBot.serviceId
            val chatBotName: String? = chatBot.name

            Lucy.d(TAG, "startSession | chatBotServiceId=$chatBotServiceId chatBotName=$chatBotName")

            if (chatBotServiceId.isNullOrEmpty() || chatBotName.isNullOrEmpty()) {
                return
            }

            if (activeSession != null) {
                if (URIUtils.compare(currentChatUri, activeSession!!.uri)) {
                    return
                }

                if (chatBotServiceId == activeSession!!.id && chatBotName == activeSession!!.name) {
                    return
                }
            }

            activeSession = CleverTapSession(currentChatUri, chatBot.serviceId, chatBot.name, System.currentTimeMillis())
            Lucy.d(TAG, "startSession | activeSession=$activeSession")
        }
    }

    private fun endSession() {
        Lucy.d(TAG, "endSession")

        synchronized(activeSessionLock) {
            if (activeSession == null) {
                Lucy.d(TAG, "endSession | No Active Session")
                return
            }

            val endedSession = activeSession!!
            activeSession = null

            endedSession.endTime = System.currentTimeMillis()

            Lucy.d(TAG, "endSession | endedSession=$endedSession")

            val sessionName = endedSession.name
            val sessionDuration = TimeUnit.MILLISECONDS.toSeconds(endedSession.duration())

            if (sessionDuration <= 0) {
                return
            }

            val sessionProperTies = HashMap<String, Any>(2)
            sessionProperTies[CleverTapValues.CHAT_BOT_SESSION_NAME] = sessionName
            sessionProperTies[CleverTapValues.CHAT_BOT_SESSION_DURATION] = sessionDuration

            sessionListener.onCleverTapSession(CleverTapValues.CHAT_BOT_SESSION_EVENT, sessionProperTies)
        }
    }

    private fun clearSession() {
        synchronized(activeSessionLock) {
            activeSession = null
        }
    }
}